import os
import yaml

from .logging import error, warn


def get(logger, filename=None):
    if filename:
        try:
            with open(filename) as f:
                return yaml.safe_load(f)
        except IOError:
            error(logger, 'config file not found at %s' % filename)

    for folder in (os.curdir,
                   os.path.join(os.path.expanduser('~'), '.dsops'),
                   os.path.join('etc', 'dsops')):
        try:
            with open(os.path.join(folder, 'dsops.yml')) as f:
                return yaml.safe_load(f)
        except IOError:
            pass

    error(logger, 'no dsops.yml file found')

def parse(logger, configfile, config, **kwargs):
    if not configfile:
        error(logger, 'config file is empty')

    app, task = kwargs['app'], kwargs['task']
    if app not in configfile:
        error(logger, '%s is not configured' % app)

    app_config = configfile[app]
    if task not in app_config:
        error(logger, '%s is not configured for %s task' % (app, task))

    task_config = app_config[task]
    for k, v in config.items():
        config[k] = task_config[k] if k in task_config else v

    if 'env' in kwargs and kwargs['env']:
        env = kwargs['env']
        if env in task_config:
            env_config = task_config[env]
            for k, v in config.items():
                config[k] = env_config[k] if k in env_config else v
        else:
            warn(logger, '%s if not configured for %s' % (app, env))

    if 'required' in kwargs:
        for key in kwargs['required']:
            if not config[key]:
                error(logger, 'no %s configured for %s' % (key, app))
