from collections import OrderedDict
import subprocess


def get(config):
    command = config.split(' ')
    lines = []

    index_hit_rate_query = \
""" SELECT SUM(idx_blks_hit) / NULLIF(SUM(idx_blks_hit + idx_blks_read), 0)
    FROM pg_statio_user_indexes;
""".replace('\n', ' ').strip()
    index_hit_rate = list(command)
    index_hit_rate.append('-c')
    index_hit_rate.append("'%s'" % index_hit_rate_query)
    data = subprocess.Popen(' '.join(index_hit_rate), shell=True,
                            stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    lines.append('index_hit_rate:%s' % data.stdout.readlines()[2])

    table_hit_rate_query = \
""" SELECT SUM(heap_blks_hit) / NULLIF(SUM(heap_blks_hit + heap_blks_read), 0)
    FROM pg_statio_user_tables;
""".replace('\n', ' ').strip()
    table_hit_rate = list(command)
    table_hit_rate.append('-c')
    table_hit_rate.append("'%s'" % table_hit_rate_query)
    data = subprocess.Popen(' '.join(table_hit_rate), shell=True,
                            stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    lines.append('table_hit_rate:%s' % data.stdout.readlines()[2])

    index_usage_query = \
""" SELECT SUM(idx_scan::float / (seq_scan + idx_scan)) / COUNT(*)
    FROM pg_stat_user_tables
    WHERE COALESCE(idx_scan, 0) <> 0;
""".replace('\n', ' ').strip()
    index_usage = list(command)
    index_usage.append('-c')
    index_usage.append("'%s'" % index_usage_query)
    data = subprocess.Popen(' '.join(index_usage), shell=True,
                            stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    lines.append('index_usage:%s' % data.stdout.readlines()[2])

    unused_indexes_query = \
""" SELECT schemaname || '.' || relname AS table,
           indexrelname AS index,
           pg_size_pretty(pg_relation_size(i.indexrelid)) AS index_size,
           idx_scan as index_scans
    FROM pg_stat_user_indexes ui
    INNER JOIN pg_index i ON ui.indexrelid = i.indexrelid
    WHERE NOT indisunique
        AND idx_scan < 50
        AND pg_relation_size(relid) > 44570;-- 5 * 8 * 1024
""".replace('\n', ' ').strip()
    unused_indexes = list(command)
    unused_indexes.append('-c')
    unused_indexes.append('"%s"' % unused_indexes_query)
    data = subprocess.Popen(' '.join(unused_indexes), shell=True,
                            stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    rows = str(len(data.stdout.readlines()) - 4)
    lines.append('unused_indexes:%s' % rows)

    num_connections_query = \
""" SELECT SUM(numbackends) FROM pg_stat_database;
""".replace('\n', ' ').strip()
    num_connections = list(command)
    num_connections.append('-c')
    num_connections.append("'%s'" % num_connections_query)
    data = subprocess.Popen(' '.join(num_connections), shell=True,
                            stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    lines.append('num_connections:%s' % data.stdout.readlines()[2])


    return lines

def keep(lines):
    keys = {
        'index_hit_rate': None,
        'table_hit_rate': None,
        'index_usage': None,
        'unused_indexes': None,
        'num_connections': None,
    }

    for line in lines:
        try:
            key, value = line.strip().split(':')
        except ValueError:
            continue

        if key in keys:
            keys[key] = value

    return {k: v for k, v in keys.items() if v}

def status(keys):
    def preprocess():
        if 'index_hit_rate' in keys:
            idxhr = keys['index_hit_rate']
            keys['index_hit_rate'] = round(float(idxhr), 3)
        if 'table_hit_rate' in keys:
            tblhr = keys['table_hit_rate']
            keys['table_hit_rate'] = round(float(tblhr), 3)
        if 'index_usage' in keys:
            idxu = keys['index_usage']
            keys['index_usage'] = round(float(idxu), 3)

    def transform(value, green=None, yellow=None, red=None):
        if green and green[0] <= value < green[1]:
            return 'GREEN'

        if yellow and yellow[0] <= value < yellow[1]:
            return 'YELLOW'

        if red and red[0] <= value < red[1]:
            return 'RED'

        return 'UNKNOWN'

    transforms = {
        'index_hit_rate': lambda x: transform(
            x, green=(0.99, float('inf')), yellow=(0.8, 0.99), red=(0, 0.8)),
        'table_hit_rate': lambda x: transform(
            x, green=(0.99, float('inf')), yellow=(0.8, 0.99), red=(0, 0.8)),
        'index_usage': lambda x: transform(
            x, green=(0.99, float('inf')), yellow=(0.8, 0.99), red=(0, 0.8)),
        'unused_indexes': lambda x: transform(
            int(x), green=(0, 1), yellow=(1, 3), red=(3, float('inf'))),
        'num_connections': lambda x: transform(
            int(x), green=(0, 70), yellow=(70, 100), red=(100, float('inf'))),
    }

    preprocess()
    for key, value in keys.items():
        try:
            keys[key] = (transforms[key](value), value)
        except KeyError:
            del keys[key]

    return OrderedDict(sorted(keys.items()))
