from collections import OrderedDict
import logging
import requests
import urlparse


def get(config):
    url = config + '/api/queues'
    parsed = urlparse.urlparse(url)
    username, password = parsed.username, parsed.password

    logging.disable(logging.warn)
    resp = requests.get(url, auth=(username, password))
    logging.disable(logging.NOTSET)

    return resp.json()

def keep(lines, queues):
    queues = {k: {} for k in queues}
    keys = {
        'messages_ready': None,
        'messages_unacknowledged': None,
        'messages': None,
        'deliver': None,
        'deliver_get': None,
        'redeliver': None,
        'consumers': None,
        'consumer_utilisation': None,
    }

    for queue in lines:
        if queue['name'] in queues:
            for key in keys:
                if key in queue:
                    value = queue[key]
                else:
                    try:
                        value = queue['message_stats'][key]
                    except KeyError:
                        value = None

                if value:
                    queues[queue['name']][key] = value

    return {k: v for k, v in queues.items() if v}

def status(keys):
    def preprocess():
        if 'messages_ready' in keys and 'messages' in keys:
            ready, total = keys['messages_ready'], keys['messages']
            ready, total = float(ready), float(total)
            if ready or total:
                keys['ready_ratio'] = round(ready / (ready + total), 3)

        if 'messages_unacknowledged' in keys and 'messages' in keys:
            unacknowledged = keys['messages_unacknowledged']
            total = keys['messages']
            unacknowledged, total = float(unacknowledged), float(total)
            if unacknowledged or total:
                keys['unacknowledged_ratio'] = round(
                    unacknowledged / (unacknowledged + total), 3)

        if 'redeliver' in keys and 'deliver' in keys:
            redeliver, deliver = keys['redeliver'], keys['deliver']
            redeliver, deliver = float(redeliver), float(deliver)
            if redeliver or deliver:
                keys['redeliver_ratio'] = round(
                    redeliver / (redeliver + deliver), 3)
        elif 'redeliver'  in keys and 'deliver_get' in keys:
            redeliver, deliver_get = keys['redeliver'], keys['deliver_get']
            redeliver, deliver_get = float(redeliver), float(deliver_get)
            if redeliver or deliver_get:
                keys['redeliver_ratio'] = round(
                    redeliver / (redeliver + deliver_get), 3)

    def transform(value, green=None, yellow=None, red=None):
        if green and green[0] <= value < green[1]:
            return 'GREEN'

        if yellow and yellow[0] <= value < yellow[1]:
            return 'YELLOW'

        if red and red[0] <= value < red[1]:
            return 'RED'

        return 'UNKNOWN'

    transforms = {
        'messages_ready': lambda x: transform(
            int(x), green=(0, 50), yellow=(50, 100), red=(100, float('inf'))),
        'unacknowledged_ratio': lambda x: transform(
            float(x), green=(0, 0.05), yellow=(0.05, 0.1),
            red=(0.1, float('inf'))),
        'redeliver_ratio': lambda x: transform(
            float(x), green=(0, 0.1), yellow=(0.1, 0.2),
            red=(0.2, float('inf'))),
        'consumers': lambda x: transform(
            int(x), green=(1, float('inf')), red=(0, 1)),
        'consumer_utilisation': lambda x: transform(
            float(x), green=(0.9, float('inf')), yellow=(0.7, 0.9),
            red=(0, 0.7)),
    }

    preprocess()
    for key, value in keys.items():
        try:
            keys[key] = (transforms[key](value), value)
        except KeyError:
            del keys[key]

    return OrderedDict(sorted(keys.items()))
