from collections import OrderedDict
import subprocess


def get(config):
    command = config.split(' ')
    command.append('info')
    data = subprocess.Popen(command, stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE)

    return data.stdout.readlines()

def keep(lines):
    keys = {
        'blocked_clients': None,
        'evicted_keys': None,
        'keyspace_hits': None,
        'keyspace_misses': None,
        'master_link_down_since_seconds': None,
        'mem_fragmentation_ratio': None,
        'rejected_connections': None,
    }

    for line in lines:
        try:
            key, value = line.strip().split(':')
        except ValueError:
            continue

        if key in keys:
            keys[key] = value

    return {k: v for k, v in keys.items() if v}

def status(keys):
    def preprocess():
        if 'keyspace_hits' in keys and 'keyspace_misses' in keys:
            hits, misses = keys['keyspace_hits'], keys['keyspace_misses']
            hits, misses = float(hits), float(misses)
            if hits or misses:
                keys['hit_ratio'] = round(hits / (hits + misses), 3)

    def transform(value, green=None, yellow=None, red=None):
        if green and green[0] <= value < green[1]:
            return 'GREEN'

        if yellow and yellow[0] <= value < yellow[1]:
            return 'YELLOW'

        if red and red[0] <= value < red[1]:
            return 'RED'

        return 'UNKNOWN'

    transforms = {
        'blocked_clients': lambda x: transform(
            int(x), green=(0, 5), yellow=(5, 20), red=(20, float('inf'))),
        'evicted_keys': lambda x: transform(
            int(x), green=(0, 1), yellow=(1, 5), red=(5, float('inf'))),
        'hit_ratio': lambda x: transform(
            float(x), green=(0.8, float('inf')), yellow=(0.6, 0.8),
            red=(0, 0.6)),
        'master_link_down_since_seconds': lambda x: transform(
            float(x), green=(0, 0), red=(0, float('inf'))),
        'mem_fragmentation_ratio': lambda x: transform(
            float(x), green=(1, 1.3), yellow=(0.8, 1.5),
            red=(float('-inf'), float('inf'))),
        'rejected_connections': lambda x: transform(
            int(x), green=(0, 5), yellow=(5, 20), red=(20, float('inf'))),
    }

    preprocess()
    for key, value in keys.items():
        try:
            keys[key] = (transforms[key](value), value)
        except KeyError:
            del keys[key]

    return OrderedDict(sorted(keys.items()))
