#!/usr/bin/env python
"""DeadSimpleStatus

Usage:
status <app> [--env=ENV] [--conf=CONF]

Options:
--env=ENV           Environment to view status of.
--conf=CONF         Specified config file.
--version           Show version.
-h --help           Show this screen.
"""
from docopt import docopt
import logging
from python_util import ColumnPrinter
import sys

from .common.config import get, parse
from .common.service import postgres, rabbitmq, redis


logger = logging.getLogger(__name__)


COLORS = {
    'GREEN': '\033[92m',
    'YELLOW': '\033[93m',
    'RED': '\033[91m',
    'BLACK': '\033[0m',
}


def output(name, status, cols=6):
    def colorize(data):
        status, value = data
        if status == 'GREEN':
            return '%s%s%s' % (COLORS[status], status, COLORS['BLACK'])

        return '%s%s - %s%s' % (COLORS[status], status, value, COLORS['BLACK'])

    num = len(status)

    print name
    print '=' * len(name)

    rows = []
    rows.append(status.keys())
    rows.append(map(colorize, status.values()))

    while num > cols:
        new_rows = [''] * 2
        rows[-1], new_rows[-1] = rows[-1][:cols], rows[-1][cols:]
        rows[-2], new_rows[-2] = rows[-2][:cols], rows[-2][cols:]
        rows.extend(new_rows)

        num -= cols

    while len(rows[-1]) < cols:
        rows[-1].append('')
        rows[-2].append('')

    printer = ColumnPrinter(prefix='= ')
    map(printer.append, rows)
    printer.output()


def main(args=None):
    if not args:
        args = docopt(__doc__, argv=sys.argv[1:],
                      version='DeadSimpleStatus v0.2.0')

    logging.basicConfig(level=logging.DEBUG,
                        format='%(asctime)s - %(name)s - %(levelname)s - '
                               '%(message)s')

    app = args.get('<app>')
    env = args.get('--env')

    conf = args.get('--conf')
    configfile = get(logger, filename=conf)
    config = {
        'postgres': None,
        'rabbitmq': None,
        'redis': None,
    }
    parse(logger, configfile, config, app=app, env=env, task='status')

    if config['postgres']:
        lines = postgres.get(config['postgres'])
        keys = postgres.keep(lines)
        status = postgres.status(keys)
        output('POSTGRES', status)

    if config['rabbitmq']:
        url, queues = config['rabbitmq']['url'], config['rabbitmq']['queues']

        lines = rabbitmq.get(url)
        keys = rabbitmq.keep(lines, queues)
        for queue in queues:
            status = rabbitmq.status(keys[queue])
            output('RABBITMQ - %s' % queue, status)

    if config['redis']:
        lines = redis.get(config['redis'])
        keys = redis.keep(lines)
        status = redis.status(keys)
        output('REDIS', status)


if __name__ == '__main__':
    main()
